import { devices } from "./hid/hid-async";
import { DEVICE_MODELS } from "./models";
export { devices, setHidAsyncType } from "./hid/hid-async";
export { StreamDeck as StreamDeckBase } from "./stream-deck";
export * from "./models";
/**
 * Select the first identified `StreamDeck`.
 * Optional vendor and product id could be used to narrow the search to a specific device.
 *
 * @param {number?} vendor optional vendor identity number
 * @param {number?} product optional product identity number
 * @returns {Promise<StreamDeck>?} the first supported `StreamDeck` within the vendor and product search space, or null
 */
export async function selectDevice(vendor, product) {
    const devs = await devices();
    for (const device of devs) {
        if (vendor === undefined || vendor === device.vendorId) {
            const models = DEVICE_MODELS[device.vendorId];
            if (models) {
                if (product === undefined || product === device.productId) {
                    const productModule = models[device.productId];
                    if (productModule) {
                        const m = await import(productModule.import);
                        return new m.default(device.path);
                    }
                    else if (product !== undefined) {
                        const m = "No implementations for the product " + product + " of vendor " + vendor;
                        const e = new Error(m);
                        e.code = "STRMDCK_MISSING_PRODUCT";
                        throw e;
                    }
                }
            }
            else if (vendor !== undefined) {
                const m = "No implementations for any product of vendor " + vendor;
                const e = new Error(m);
                e.code = "STRMDCK_MISSING_VENDOR";
                throw e;
            }
        }
    }
    return null;
}
/**
 * Select all the identified `StreamDeck`s.
 * Optional vendor and product id could be used to narrow the search to range of devices.
 *
 * @param {number?} vendor optional vendor identity number
 * @param {number?} product optional product identity number
 * @returns {Array<Promise<StreamDeck>>} a list of promises to `StreamDeck`s within the vendor and product search space
 */
export async function selectAllDevices(vendor, product) {
    const res = [];
    const devs = await devices();
    for (const device of devs) {
        if (vendor === undefined || vendor === device.vendorId) {
            const models = DEVICE_MODELS[device.vendorId];
            if (models) {
                if (product === undefined || product === device.productId) {
                    const productModule = models[device.productId];
                    if (productModule) {
                        res.push(import(productModule.import).then((m) => new m.default(device.path)));
                    }
                    else if (product !== undefined) {
                        const m = "No implementations for the product " + product + " of vendor " + vendor;
                        const e = new Error(m);
                        e.code = "STRMDCK_MISSING_PRODUCT";
                        throw e;
                    }
                }
            }
            else if (vendor !== undefined) {
                const m = "No implementations for any product of vendor " + vendor;
                const e = new Error(m);
                e.code = "STRMDCK_MISSING_VENDOR";
                throw e;
            }
        }
    }
    return res;
}
//# sourceMappingURL=exports.js.map