import { EventEmitter } from "./event-emitter";
import { HidAsync } from "./hid/hid-async";
import { IImageLibrary, IImageLibraryCreator, IImageRawOptions } from "./image-library";
/**
 * Abstract base class which is a partial implementation and must be extended.
 *
 * To see which abstract properties and methods are required for a subclass
 * check out the typings and look for the keyword `abstract`.
 */
export declare abstract class StreamDeck extends EventEmitter {
    /**
     * The pixel size of an icon written to the Stream Deck key.
     *
     * @readonly
     */
    abstract readonly iconSize: number;
    /**
     * The number of keys available for the Stream Deck.
     *
     * @readonly
     */
    abstract readonly buttonLength: number;
    /**
     * The number of columns of keys available for the Stream Deck.
     *
     * @readonly
     */
    abstract readonly buttonColumns: number;
    /**
     * The number of rows of keys available for the Stream Deck.
     *
     * @readonly
     */
    abstract readonly buttonRows: number;
    /**
     * The layout of the stream deck showing which position has a specific `keyIndex`.
     *
     * @readonly
     */
    abstract readonly buttonLayout: number[][];
    /**
     * The size of a page packet.
     *
     * @readonly
     */
    protected abstract readonly pagePacketSize: number;
    /**
     * The connected device.
     */
    protected device: HidAsync;
    /**
     * The current state of key presses.
     *
     * Limited to 32 buttons.
     */
    protected keyState: number;
    protected _imageLibrary?: string | IImageLibraryCreator | PromiseLike<IImageLibraryCreator>;
    /**
     * The image library to use.
     * If not already imported getting this property will trigger the import.
     *
     * @readonly
     */
    readonly imageLibrary: IImageLibraryCreator | PromiseLike<IImageLibraryCreator>;
    /**
     * A sorted list containing the `keyIndex` of all buttons currently pressed.
     *
     * @readonly
     */
    readonly pressedKeys: number[];
    /**
     * A boolean showing if there currently are any pressed keys.
     *
     * @readonly
     */
    readonly hasPressedKeys: boolean;
    /**
     * Connects to a HID device.
     *
     * @param {string} devicePath path to the HID device.
     */
    constructor(devicePath: string);
    /**
     * Sets which image library should be used.
     *
     * If the parameter is a `string` the module or file will be imported when needed
     * and expects `default` to be an `IImageLibraryCreator`.
     * If no `default` is exported the functions defined in an `IImageLibraryCreator` must have been exported directly.
     *
     * @param {string | Promise<Object> | Object} library A name, path, or reference to an image library.
     * @return {StreamDeck} this
     */
    setImageLibrary(library: string | IImageLibraryCreator | PromiseLike<IImageLibraryCreator>): this;
    /**
     * Checks a keyIndex is a valid key for a stream deck. An integer between 0 and `buttonLength - 1`.
     *
     * @param {number} keyIndex The keyIndex to check
     */
    checkValidKeyIndex(keyIndex: number): number;
    /**
     * Loops through every available keyIndex.
     *
     * @param {Function} func The function to pass the keyIndex and the streamDeck instance to
     * @returns {StreamDeck} this
     */
    forEachKey(func: (keyIndex: number, streamDeck: this) => any): this;
    /**
     * Get the `keyIndex` at a specific column and row.
     *
     * @param {number} x column number
     * @param {number} y row number
     * @returns {number} keyIndex
     */
    buttonIndexFromPosition(x: number, y: number): number | undefined;
    /**
     * Writes a Buffer to the Stream Deck.
     *
     * @param {Uint8Array} buffer The buffer written to the Stream Deck
     * @returns {Promise<number>} Resolves to the number of bytes written
     */
    write(buffer: Uint8Array): Promise<number>;
    /**
     * Writes a Buffer to the Stream Deck.
     *
     * @param {Uint8Array[]} buffers A list of buffers to be written to the Stream Deck
     * @returns {Promise<number>} Resolves to the number of bytes written
     */
    writeMulti(buffers: Uint8Array[]): Promise<number>;
    /**
     * Sends a HID feature report to the Stream Deck.
     *
     * @param {Uint8Array} buffer The buffer send to the Stream Deck.
     * @returns {Promise<number>} Resolves to the number of bytes written
     */
    sendFeatureReport(buffer: Uint8Array): Promise<number>;
    /**
     * Closes this reference to the device.
     */
    close(): void;
    /**
     * Fills the given key with a solid color.
     *
     * @param {number} keyIndex The key to fill 0 - 14
     * @param {number} r The color's red value. 0 - 255
     * @param {number?} g The color's green value. 0 - 255
     * @param {number?} b The color's blue value. 0 -255
     * @return {Promise<number>} Resolves to the number of bytes written.
     */
    fillColor(keyIndex: number, rgb: number): Promise<number>;
    fillColor(keyIndex: number, r: number, g: number, b: number): Promise<number>;
    /**
     * Fills the given key with an image in a Buffer.
     *
     * @param {number} keyIndex The key to fill 0 - 14
     * @param {Uint8Array} imageBuffer
     */
    fillImage(keyIndex: number, imageBuffer: Uint8Array): Promise<number>;
    /**
     * Fills the given key with an image from a file.
     *
     * @param {number} keyIndex The key to fill 0 - 14
     * @param {string} filePath A file path to an image file
     * @returns {Promise<number>} Resolves to the number of bytes that has been written
     */
    fillImageFromFile(keyIndex: number, filePath: string): Promise<number>;
    /**
     * Fills the whole panel with an image in a Buffer.
     * The image is scaled to fit, and then center-cropped (if necessary).
     *
     * @param {Uint8Array|string} imagePathOrBuffer
     * @param {Object} [rawOptions] If supplying a buffer of raw pixels the size of the image must be specified.
     * @param {Object} [rawOptions.channels] Number of channels per pixel.
     * @param {Object} [rawOptions.height] Height of the raw pixel image.
     * @param {Object} [rawOptions.width] Width of the raw pixel image.
     */
    fillPanel(imagePathOrBuffer: Uint8Array | string): Promise<number>;
    fillPanel(imagePathOrBuffer: Uint8Array, rawOptions: IImageRawOptions): Promise<number>;
    /**
     * Clears the given key.
     *
     * @param {number} keyIndex The key to clear 0 - 14
     */
    clearKey(keyIndex: number): Promise<number>;
    /**
     * Clears all keys.
     *
     * returns {undefined}
     */
    clearAllKeys(): this;
    /**
     * Sets the brightness of the keys on the Stream Deck
     *
     * @param {number} percentage The percentage brightness
     */
    setBrightness(percentage: number): Promise<number>;
    protected onDeviceData(data: Uint8Array): void;
    /**
     * Process an image to be shown on the stream deck.
     *
     * @param {Object} image image to extract pixels from.
     */
    protected processImage(image: IImageLibrary): Promise<Uint8Array>;
    protected abstract writeImagePage(keyIndex: number, pixels: Uint8Array): Promise<number>;
}
