"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const event_emitter_1 = require("./event-emitter");
const helpers_1 = require("./helpers");
const hid_async_1 = require("./hid/hid-async");
/**
 * Abstract base class which is a partial implementation and must be extended.
 *
 * To see which abstract properties and methods are required for a subclass
 * check out the typings and look for the keyword `abstract`.
 */
class StreamDeck extends event_emitter_1.EventEmitter {
    /**
     * The image library to use.
     * If not already imported getting this property will trigger the import.
     *
     * @readonly
     */
    get imageLibrary() {
        if (!this._imageLibrary) {
            this._imageLibrary = Promise.resolve().then(() => __importStar(require("./image-library"))).then((x) => x.default);
        }
        else if (typeof this._imageLibrary === "string") {
            this._imageLibrary = Promise.resolve().then(() => __importStar(require(this._imageLibrary))).then((x) => x.default || x);
        }
        return this._imageLibrary;
    }
    /**
     * A sorted list containing the `keyIndex` of all buttons currently pressed.
     *
     * @readonly
     */
    get pressedKeys() {
        const r = [];
        for (let i = 0; i < this.buttonLength; i++) {
            // tslint:disable-next-line:no-bitwise
            if (this.keyState & (1 << i)) {
                r.push(i);
            }
        }
        return r;
    }
    /**
     * A boolean showing if there currently are any pressed keys.
     *
     * @readonly
     */
    get hasPressedKeys() {
        return this.keyState !== 0;
    }
    /**
     * Connects to a HID device.
     *
     * @param {string} devicePath path to the HID device.
     */
    constructor(devicePath) {
        super();
        if (!devicePath) {
            throw new Error("No Stream Decks are connected.");
        }
        else {
            this.device = new hid_async_1.HidAsync(devicePath);
        }
        this.keyState = 0;
        this.device.on("data", (data) => this.onDeviceData(data));
        this.device.on("error", (err) => {
            if (typeof err !== "object") {
                err = new Error(err);
            }
            if (!this[event_emitter_1.emit]("error", err)) {
                Promise.resolve().then(() => {
                    throw err;
                });
            }
        });
    }
    /**
     * Sets which image library should be used.
     *
     * If the parameter is a `string` the module or file will be imported when needed
     * and expects `default` to be an `IImageLibraryCreator`.
     * If no `default` is exported the functions defined in an `IImageLibraryCreator` must have been exported directly.
     *
     * @param {string | Promise<Object> | Object} library A name, path, or reference to an image library.
     * @return {StreamDeck} this
     */
    setImageLibrary(library) {
        this._imageLibrary = library;
        return this;
    }
    /**
     * Checks a keyIndex is a valid key for a stream deck. An integer between 0 and `buttonLength - 1`.
     *
     * @param {number} keyIndex The keyIndex to check
     */
    checkValidKeyIndex(keyIndex) {
        // tslint:disable-next-line:no-bitwise
        if (typeof keyIndex !== "number" || (keyIndex | 0) !== keyIndex) {
            throw new TypeError("Expected keyIndex to be an integer");
        }
        if (keyIndex < 0 || keyIndex >= this.buttonLength) {
            throw new TypeError("Expected a valid keyIndex 0 - " + (this.buttonLength - 1));
        }
        return keyIndex;
    }
    /**
     * Loops through every available keyIndex.
     *
     * @param {Function} func The function to pass the keyIndex and the streamDeck instance to
     * @returns {StreamDeck} this
     */
    forEachKey(func) {
        for (let i = 0; i < this.buttonLength; i++) {
            func.call(this, i, this);
        }
        return this;
    }
    /**
     * Get the `keyIndex` at a specific column and row.
     *
     * @param {number} x column number
     * @param {number} y row number
     * @returns {number} keyIndex
     */
    buttonIndexFromPosition(x, y) {
        if (this.buttonLayout[y] === undefined) {
            return undefined;
        }
        return this.buttonLayout[y][x];
    }
    /**
     * Writes a Buffer to the Stream Deck.
     *
     * @param {Uint8Array} buffer The buffer written to the Stream Deck
     * @returns {Promise<number>} Resolves to the number of bytes written
     */
    write(buffer) {
        return this.device.write(buffer);
    }
    /**
     * Writes a Buffer to the Stream Deck.
     *
     * @param {Uint8Array[]} buffers A list of buffers to be written to the Stream Deck
     * @returns {Promise<number>} Resolves to the number of bytes written
     */
    writeMulti(buffers) {
        return this.device.writeMulti(buffers);
    }
    /**
     * Sends a HID feature report to the Stream Deck.
     *
     * @param {Uint8Array} buffer The buffer send to the Stream Deck.
     * @returns {Promise<number>} Resolves to the number of bytes written
     */
    sendFeatureReport(buffer) {
        return this.device.sendFeatureReport(buffer);
    }
    /**
     * Closes this reference to the device.
     */
    close() {
        this.device.close();
    }
    fillColor(keyIndex, r, g, b) {
        this.checkValidKeyIndex(keyIndex);
        if (g === undefined || b === undefined) {
            // tslint:disable-next-line:no-bitwise
            b = r & 0x0000FF;
            // tslint:disable-next-line:no-bitwise
            g = (r & 0x00FF00) >> 8;
            // tslint:disable-next-line:no-bitwise
            r = (r & 0xFF0000) >> 16;
        }
        helpers_1.checkRGBValue(r);
        helpers_1.checkRGBValue(g);
        helpers_1.checkRGBValue(b);
        const pixels = Buffer.alloc(this.iconSize * this.iconSize * 3, Buffer.from([b, g, r]));
        return this.writeImagePage(keyIndex, pixels);
    }
    /**
     * Fills the given key with an image in a Buffer.
     *
     * @param {number} keyIndex The key to fill 0 - 14
     * @param {Uint8Array} imageBuffer
     */
    fillImage(keyIndex, imageBuffer) {
        this.checkValidKeyIndex(keyIndex);
        const iconWidth = this.iconSize;
        const iconHeight = this.iconSize;
        const area = iconWidth * iconHeight;
        const areadepth = area * 3;
        if (imageBuffer.length !== areadepth) {
            throw new RangeError(`Expected image buffer of length ${areadepth}, got length ${imageBuffer.length}`);
        }
        const pixels = Buffer.alloc(area * 3);
        const l = iconWidth * 3;
        for (let y = 0; y < iconHeight; y++) {
            const start = y * l;
            for (let i = 0; i < l; i += 3) {
                const r = imageBuffer[i + start];
                const g = imageBuffer[i + start + 1];
                const b = imageBuffer[i + start + 2];
                pixels[start + l - i - 1] = r && 0x80;
                pixels[start + l - i - 2] = g;
                pixels[start + l - i - 3] = b && 0x20;
            }
        }
        return this.writeImagePage(keyIndex, pixels);
    }
    /**
     * Fills the given key with an image from a file.
     *
     * @param {number} keyIndex The key to fill 0 - 14
     * @param {string} filePath A file path to an image file
     * @returns {Promise<number>} Resolves to the number of bytes that has been written
     */
    async fillImageFromFile(keyIndex, filePath) {
        this.checkValidKeyIndex(keyIndex);
        const imglib = await this.imageLibrary;
        const imgins = await imglib.loadFile(filePath);
        const buffer = await this.processImage(imgins);
        return this.fillImage(keyIndex, buffer);
    }
    async fillPanel(imagePathOrBuffer, rawOptions) {
        const iconHeight = this.iconSize;
        const iconWidth = this.iconSize;
        const cols = this.buttonColumns;
        const imglib = await this.imageLibrary;
        const image0 = await (rawOptions
            ? imglib.createRaw(imagePathOrBuffer, rawOptions)
            : typeof imagePathOrBuffer === "string"
                ? imglib.loadFile(imagePathOrBuffer)
                : imglib.loadFileData(imagePathOrBuffer));
        const image1 = await image0.resize(cols * iconWidth, this.buttonRows * iconHeight);
        const image = await image1.flatten(); // Eliminate alpha channel, if any.
        const buttons = [];
        for (let row = 0; row < this.buttonRows; row++) {
            for (let column = 0; column < cols; column++) {
                buttons.push({
                    index: (row * cols) + cols - column - 1,
                    x: column,
                    y: row,
                });
            }
        }
        const ret = [];
        for (const button of buttons) {
            const part = await image.extract({
                height: iconHeight,
                left: button.x * iconWidth,
                top: button.y * iconHeight,
                width: iconWidth,
            });
            ret.push(this.fillImage(button.index, await part.toUint8Array()));
        }
        return Promise.all(ret).then((a) => a.reduce((x, y) => x + y, 0));
    }
    /**
     * Clears the given key.
     *
     * @param {number} keyIndex The key to clear 0 - 14
     */
    clearKey(keyIndex) {
        this.checkValidKeyIndex(keyIndex);
        return this.fillColor(keyIndex, 0, 0, 0);
    }
    /**
     * Clears all keys.
     *
     * returns {undefined}
     */
    clearAllKeys() {
        return this.forEachKey(this.clearKey);
    }
    /**
     * Sets the brightness of the keys on the Stream Deck
     *
     * @param {number} percentage The percentage brightness
     */
    setBrightness(percentage) {
        if (percentage < 0 || percentage > 100) {
            throw new RangeError("Expected brightness percentage to be between 0 and 100");
        }
        const brightnessCommandBuffer = Buffer.alloc(17);
        brightnessCommandBuffer.set([0x05, 0x55, 0xaa, 0xd1, 0x01, percentage]);
        return this.sendFeatureReport(brightnessCommandBuffer);
    }
    onDeviceData(data) {
        // The first byte is a report ID, the last byte appears to be padding.
        // We strip these out for now.
        data = data.slice(1, data.length - 1);
        for (let i = 0; i < this.buttonLength; i++) {
            // tslint:disable-next-line:no-bitwise
            const bitidx = 1 << i;
            const keyPressed = data[i] ? bitidx : 0;
            // tslint:disable-next-line:no-bitwise
            const stateChanged = keyPressed !== (this.keyState & bitidx);
            if (stateChanged) {
                // tslint:disable-next-line:no-bitwise
                this.keyState ^= bitidx;
                if (keyPressed) {
                    this[event_emitter_1.emit]("down", i);
                }
                else {
                    this[event_emitter_1.emit]("up", i);
                }
            }
        }
    }
    /**
     * Process an image to be shown on the stream deck.
     *
     * @param {Object} image image to extract pixels from.
     */
    async processImage(image) {
        const i0 = await image.flatten(); // Eliminate alpha channel, if any.
        const i1 = await i0.resize(this.iconSize, this.iconSize);
        return i1.toUint8Array();
    }
}
exports.StreamDeck = StreamDeck;
//# sourceMappingURL=stream-deck.js.map